%%% FIG_DATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
fig_dir  = ['output' filesep 'figures' filesep];

% Load and arrange data
datasets  = load([data_dir 'data.mat']);
time      = datetime(datenum(datasets.GDE.time, 'yyyyQQ'), 'convertfrom', 'datenum');
subsample = (datetime(1984, 1, 1) <= time) & (time < datetime(2026, 1, 1));
time      = time(subsample, :); 
Y_GDE     = 100*log( [datasets.GDE.early(subsample, :), ...
                      datasets.GDE.late(subsample, :), ...
                      datasets.GDE.compr(subsample, :)] )';
Y_GDI     = 100*log( [datasets.GDI.early(subsample, :), ...
                      datasets.GDI.late(subsample, :), ...
                      datasets.GDI.compr(subsample, :)] )';

% Compute an store dimensions in a vector
N_c  = length(datasets.GDE.compr_aux);
N_E  = size(Y_GDE, 1);
N_I  = size(Y_GDI, 1);
T    = nnz(subsample);
dims = [N_c, N_E, N_I, T];
                   
% Define vintage indicators    
date_vintage = vintage2date( [datasets.GDE.early_vintage(subsample, :), ...
                              datasets.GDE.late_vintage(subsample, :), ...
                              datasets.GDE.compr_vintage(subsample, :), ...
                              datasets.GDI.early_vintage(subsample, :), ...
                              datasets.GDI.late_vintage(subsample, :), ...
                              datasets.GDI.compr_vintage(subsample, :)]' );

% Define signal indicators
delta_signal = NaN(N_E+N_I, T);
date_signal  = vintage2date( datasets.GDE.compr_aux );
year_signals = [1998, 2003, 2009, 2013, 2018];
for i_c = 1:(N_c-1)
    delta_signal(date_vintage >= date_signal(i_c) & date_vintage < date_signal(i_c+1)) = i_c;
end
delta_signal(date_vintage >= date_signal(N_c)) = N_c;
delta_signal = fillmissing(delta_signal, 'next', 2);     % impute missing entries of old signals using next available
delta_signal = fillmissing(delta_signal, 'previous', 2); % impute missing entries of new/discontinued signals using previous available

% Store indicators in 3D array
delta = zeros(N_E+N_I, N_c, T);
for i_c = 1:N_c
    delta(:, i_c, :) = double(delta_signal == i_c);
end

% Inputs for plots and tables
ax_fonts   = {'fontname', 'times', 'fontsize', 10, 'linewidth', 0.5};
blue       = [ 51,  51, 153]/255;
red        = [153,  51,  51]/255;
green      = [ 51, 153,  51]/255;
dark_green = [  0, 100,   0]/255;


%% ALL REVISIONS

fig0 = figure();
ax0  = axes();
for i_c = 1:N_c

    % Upload GDE and GDI data
    y_E     = Y_GDE;
    y_I     = Y_GDI;
    delta_E = reshape(delta(1:N_E, i_c, :), [N_E, T]);
    delta_I = reshape(delta(N_E+(1:N_I), i_c, :), [N_I, T]);
    
    % Discard empty series in GDE
    [n_E, T] = size(y_E);
    y_E_keep = NaN(n_E, T);
    for i = 1:n_E, for t = 1:T, if delta_E(i, t), y_E_keep(i, t) = y_E(i, t); end; end; end
    keep_E   = ~all(isnan(y_E_keep), 2);
    y_E      = y_E_keep(keep_E, :);
    Dy_E     = exp(y_E(:, 2:end)/100)';
    n_E      = size(y_E, 1);
    
    % Discard empty series in GDI
    [n_I, T] = size(y_I);
    y_I_keep = NaN(n_I, T);
    for i = 1:n_I, for t = 1:T, if delta_I(i, t), y_I_keep(i, t) = y_I(i, t); end; end; end
    keep_I   = ~all(isnan(y_I_keep), 2);
    y_I      = y_I_keep(keep_I, :);
    Dy_I     = exp(y_I(:, 2:end)/100)';
    n_I      = size(y_I, 1);
    
    % Create style and color property cells
    colors          = cell(n_E+n_I, 1);
    for i = 1:n_E,       colors{i} = blue; end
    for i = n_E+(1:n_I), colors{i} = red;    end
    styles          = cell(n_E+n_I, 1);
    for i = 1:min(3, (n_E-1)),   styles{i} = ':';  end
    for i = min(4, n_E):(n_E-1), styles{i} = '-.'; end
    styles{n_E}     = '-';
    for i = n_E+(1:min(3, (n_I-1))),   styles{i} = ':';  end
    for i = n_E+(min(4, n_I):(n_I-1)), styles{i} = '-.'; end
    styles{n_E+n_I} = '-';
    
    % Create figure
    plot0 = plot(ax0, time(2:end), [Dy_E, Dy_I]);
    hold on 
    
    % Add extras
    xlim(ax0, [time(2), time(end)])
    ylim(ax0, [0.5e4, 2.1e4])
    ylabel(ax0, 'GDE (blue) and GDI (red)')
    
    % Add label
    i_aux = find(year(time(2:end)) == year_signals(i_c), 1, 'first');
    text(ax0, time(i_aux), 0.97*mean([Dy_E(i_aux, :), Dy_I(i_aux, :)], 'omitnan'), ...
        sprintf('%d', year_signals(i_c)), ax_fonts{:})
    
    % Tune ax handle
    set(ax0, ax_fonts{:})
    set(ax0, 'box', 'on')
    
    % Tune plot
    set(plot0, 'linewidth', 2)
    set(plot0, {'linestyle'}, styles)
    set(plot0, {'color'}, colors)
    
end

% Save figure
hold off
print(fig0, [fig_dir 'data_all.eps'], '-depsc')


%% 2015-2017

% Arrange data to plot
time_plot = (2015 <= year(time)) & (year(time) <= 2017);
Y_GDE_aux = Y_GDE(any(~isnan(Y_GDE(:, time_plot)), 2), time_plot);
Y_GDI_aux = Y_GDI(any(~isnan(Y_GDI(:, time_plot)), 2), time_plot);
Y_GDE_aux = Y_GDE_aux(end:(-1):1, :);
Y_GDI_aux = Y_GDI_aux(end:(-1):1, :);

% Create figure for zoomed-in data
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time(time_plot), exp([Y_GDE_aux', Y_GDI_aux']/100));

% Create style and color property cells
colors  = {blue; blue; blue; blue; blue; blue;...
                  red;  red;  red;  red;  red};
styles  = {'-'; ':'; '-.'; '--'; '-'; '-';...
           '-'; ':'; '-.'; '--'; '-'};
markers = {'s'; 'x'; 'd'; '+'; 'o'; 'p';...
           's'; 'x'; 'd'; '+'; 'o'};

% Change x-axis to quarters
xlim(ax0, [datetime(2014, 12, 1), datetime(2017, 12, 1)])
xticks(ax0, datetime(2015, 4+6*(0:5), 1))
xticklabels(ax0, {'2015Q2', '2015Q4', '2016Q2', '2016Q4', '2017Q2', '2017Q4'})

% Add extras
ylabel(ax0, 'GDE (blue) and GDI (red)')
try
    legend(ax0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                 'annual 3', 'annual 2', 'annual 1', 'third', 'second'}, ...
                 'location', 'SouthEast', 'NumColumns', 2) %#ok
catch
    legend(ax0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                 'annual 3', 'annual 2', 'annual 1', 'third', 'second'}, ...
                 'location', 'best')    
end

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, styles)
set(plot0, {'color'}, colors)
set(plot0, {'marker'}, markers)
set(plot0, 'markersize', 5)

% Save figure
print(fig0, [fig_dir 'data_2015_2017.eps'], '-depsc')


%% 2019-2021

% Arrange data to plot
time_plot = (2019 <= year(time)) & (year(time) <= 2021);
Y_GDE_aux = Y_GDE(any(~isnan(Y_GDE(:, time_plot)), 2), time_plot);
Y_GDI_aux = Y_GDI(any(~isnan(Y_GDI(:, time_plot)), 2), time_plot);
Y_GDE_aux = Y_GDE_aux(end:(-1):1, :);
Y_GDI_aux = Y_GDI_aux(end:(-1):1, :);

% Create figure for zoomed-in data
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time(time_plot), exp([Y_GDE_aux', Y_GDI_aux']/100));

% Create style and color property cells
colors  = {blue; blue; blue; blue; blue; blue;...
                  red;  red;  red;  red;  red};
styles  = {'-'; ':'; '-.'; '--'; '-'; '-';...
           '-'; ':'; '-.'; '--'; '-'};
markers = {'s'; 'x'; 'd'; '+'; 'o'; 'p';...
           's'; 'x'; 'd'; '+'; 'o'};

% Change x-axis to quarters
xlim(ax0, [datetime(2018, 12, 1), datetime(2021, 12, 1)])
xticks(ax0, datetime(2019, 4+6*(0:5), 1))
xticklabels(ax0, {'2019Q2', '2019Q4', '2020Q2', '2020Q4', '2021Q2', '2021Q4'})
       
% Add extras
ylabel(ax0, 'GDE (blue) and GDI (red)')
try
    legend(ax0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                 'annual 3', 'annual 2', 'annual 1', 'third', 'second'}, ...
                 'location', 'SouthEast', 'NumColumns', 2) %#ok
catch
    legend(ax0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                 'annual 3', 'annual 2', 'annual 1', 'third', 'second'}, ...
                 'location', 'best')    
end

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, styles)
set(plot0, {'color'}, colors)
set(plot0, {'marker'}, markers)
set(plot0, 'markersize', 5)

% Save figure
print(fig0, [fig_dir 'data_2019_2021.eps'], '-depsc')


%% 2019-2022

% Arrange data to plot
time_plot = (2019 <= year(time)) & (year(time) <= 2023);
Y_GDE_aux = Y_GDE(any(~isnan(Y_GDE(:, time_plot)), 2), time_plot);
Y_GDI_aux = Y_GDI(any(~isnan(Y_GDI(:, time_plot)), 2), time_plot);
Y_GDE_aux = Y_GDE_aux(end:(-1):1, :);
Y_GDI_aux = Y_GDI_aux(end:(-1):1, :);

% Create figure for zoomed-in data
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time(time_plot), exp([Y_GDE_aux', Y_GDI_aux']/100));

% Create style and color property cells
colors  = {blue; blue; blue; blue; blue; blue;...
                  red;  red;  red;  red;  red};
styles  = {'-'; ':'; '-.'; '--'; '-'; '-';...
           '-'; ':'; '-.'; '--'; '-'};
markers = {'s'; 'x'; 'd'; '+'; 'o'; 'p';...
           's'; 'x'; 'd'; '+'; 'o'};

% Change x-axis to quarters
xlim(ax0, [datetime(2018, 12, 1), datetime(2022, 12, 1)])
xticks(ax0, datetime(2019, 4+6*(0:7), 1))
xticklabels(ax0, {'2019Q2', '2019Q4', '2020Q2', '2020Q4', '2021Q2', '2021Q4', '2022Q2', '2022Q4'})
       
% Add extras
ylabel(ax0, 'GDE (blue) and GDI (red)')
try
    legend(ax0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                 'annual 3', 'annual 2', 'annual 1', 'third', 'second'}, ...
                 'location', 'SouthEast', 'NumColumns', 2) %#ok
catch
    legend(ax0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                 'annual 3', 'annual 2', 'annual 1', 'third', 'second'}, ...
                 'location', 'best')    
end

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, styles)
set(plot0, {'color'}, colors)
set(plot0, {'marker'}, markers)
set(plot0, 'markersize', 5)

% Save figure
print(fig0, [fig_dir 'data_2019_2022.eps'], '-depsc')
