%%% FIG_FULL_SAMPLE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set filename
fname = '';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
fig_dir  = ['output' filesep 'figures' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];
tab_dir  = ['output' filesep 'tables' filesep];

% Recover estimates
results      = load([mat_dir sprintf('estimates_full_sample%s.mat', fname)]);
settings     = results.settings;
time         = results.time;
ref_vintage  = time(end) + calmonths(12);
date_vintage = results.date_vintage;
Y            = results.Y;
dims         = results.dims;
delta        = results.delta;
prior        = results.prior;
param_Gibbs  = results.param_Gibbs;

% Extract dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);
T   = dims(4);
[n_param, n_MC] = size(param_Gibbs);

% Inputs for plots and tables
ax_fonts   = {'fontname', 'times', 'fontsize', 10, 'linewidth', 0.5};
black      = [  0,   0,   0];
blue       = [ 51,  51, 153]/255;
red        = [153,  51,  51]/255;
green      = [ 51, 153,  51]/255;
dark_green = [  0, 100,   0]/255;


%% TABLES AND PLOTS OF POSTERIORS

% Fix seed for random number generator
rng(1918)

% Simulate from the prior
param_prior       = NaN(n_param, settings.n_MC);
for i_MC = 1:settings.n_MC
    param_prior(:, i_MC) = simulate_prior(prior);
end

% Recover description
description = name_parameter(dims);

% Compute Monte Carlo standard errors for the mean
param_MCSE = NaN(n_param, 2);
for i_param = 1:n_param
    param_MCSE(i_param, 1) = sqrt(compute_LRV(param_Gibbs(i_param, :))/n_MC);    
    param_MCSE(i_param, 2) = corr(param_Gibbs(i_param, 2:n_MC)', param_Gibbs(i_param, 1:(n_MC-1))');    
end

% Display results in a table
table_param       = array2table([round(quantile(param_prior, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(quantile(param_Gibbs, [0.05, 0.50, 0.95], 2), 3), ...
                                 round(param_MCSE(:, 1), 4), round(param_MCSE(:, 2), 2)], ...
    'variablenames', {'LowPrior', 'MedPrior', 'HighPrior', 'LowPost', 'MedPost', 'HighPost', 'MCSE', 'Autocorr'});
table_param.Names = description.param_names_tab;
table_param       = [table_param(:, 'Names'), table_param(:, 1:(end-1))];
display(table_param)
writetable(table_param, [tab_dir sprintf('posteriors%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)

% Display densities in a plot
n_abs = 1e2;
for i_param = 1:length(description.param_groupnames)
    
    n_G  = description.n_param(i_param);
    fig0 = figure();
    set(fig0, 'units', 'normalized', 'position', [0 0 1 1])
    
    if (n_G <= 121)
        
        n_G_tmp        = sum(description.n_param(1:(i_param-1)), 2);
        [n_row, n_col] = compute_dimensions(n_G);
        
        for i_G = 1:n_G
            
            % Compute plot data
            abscisa       = linspace(quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.005, 2), ...
                quantile([param_prior(n_G_tmp+i_G, :), param_Gibbs(n_G_tmp+i_G, :)], 0.995, 2), n_abs)';
            density_prior = ksdensity(param_prior(n_G_tmp+i_G, :)', abscisa);
            density_Gibbs = ksdensity(param_Gibbs(n_G_tmp+i_G, :)', abscisa);
            
            % Execute and format plot
            ax0   = subplot(n_row, n_col, i_G);
            hold(ax0, 'all')
            plot0 = plot(ax0, abscisa, [density_prior, density_Gibbs]);
            area(ax0, abscisa, density_prior, 'facecolor', green, 'facealpha', 0.2)
            area(ax0, abscisa, density_Gibbs, 'facecolor', dark_green, 'facealpha', 0.5)
            hold(ax0, 'off')
            set(ax0, ax_fonts{:})
            set(ax0, 'box', 'on')
            axis(ax0, 'tight')
            ylabel(ax0, description.param_names_fig{n_G_tmp+i_G, 1}, 'Interpreter', 'TeX')
            set(plot0, {'color'}, {green; dark_green})

        end
                
    end
    
    print(fig0, [fig_dir sprintf('posteriors_%s.eps', description.param_groupnames{i_param})], '-deps')
    
end


%% PLOTS OF GDP SOLERA VS LAST-SIGNAL/LAST-SERIES

% Load GDE and GDI data
GDE_data   = readtable([data_dir 'GDE_raw.csv']);
GDE_date   = cellfun(@(s) datetime(str2double(s(1:4)), 3*str2double(s(6))-2, 1), GDE_data{:, 1});
growth_GDE = GDE_data{:, end};
growth_GDE = [NaN(1, size(growth_GDE, 2)); 100*((growth_GDE(2:end, :)./growth_GDE(1:(end-1), :)).^4-1)];
growth_GDE = growth_GDE((GDE_date >= min(time)) & (GDE_date <= max(time)), :);
GDI_data   = readtable([data_dir 'GDI_raw.csv']);
GDI_date   = cellfun(@(s) datetime(str2double(s(1:4)), 3*str2double(s(6))-2, 1), GDI_data{:, 1});
growth_GDI = GDI_data{:, end};
growth_GDI = [NaN(1, size(growth_GDI, 2)); 100*((growth_GDI(2:end, :)./growth_GDI(1:(end-1), :)).^4-1)];
growth_GDI = growth_GDI((GDI_date >= min(time)) & (GDI_date <= max(time)), :);

% Compute latest GDE and GDI series
mat_vintage = (date_vintage <= ref_vintage);
GDE_tmp     = NaN(1, T);
GDI_tmp     = NaN(1, T);
for t = 1:T
    GDE_aux = find(mat_vintage(1:N_E, t), 1, 'last');
    if ~isempty(GDE_aux), GDE_tmp(t) = GDE_aux; else, GDE_tmp(t) = false; end
    GDI_aux = N_E + find(mat_vintage(N_E+(1:N_I), t), 1, 'last');
    if ~isempty(GDI_aux), GDI_tmp(t) = GDI_aux; else, GDI_tmp(t) = false; end
end

% Compute dataset with last signal only
vintage_tmp                = logical(squeeze(delta(:, N_c, :)));
Y_last_signal              = NaN(N_E+N_I, T);
Y_last_signal(vintage_tmp) = Y(vintage_tmp);

% Compute dataset with last series only
vintage_tmp = false(N_E+N_I, T);
for t = 1:T
    if (GDE_tmp(t)>0), vintage_tmp(GDE_tmp(t), t) = true; end
    if (GDI_tmp(t)>0), vintage_tmp(GDI_tmp(t), t) = true; end
end
Y_last_series              = NaN(N_E+N_I, T);
Y_last_series(vintage_tmp) = Y(vintage_tmp);

% Compute smoothed states for last-series and last-signal data
n_MC            = settings.n_MC;
GDP_solera      = NaN(N_c, T, n_MC);
GDP_last_series = NaN(N_c, T, n_MC);
GDP_last_signal = NaN(N_c, T, n_MC);
mu              = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
SSM_med         = construct_SSM(median(param_Gibbs, 2), delta, dims, mu);
del_string      = '';
avg_time        = NaN;
tic
for i_MC = 1:n_MC
    % Report progress
    message    = sprintf('Smoothing states - Draw: %d/%d - Time left: %d mins %02d secs\n', i_MC, n_MC, floor(avg_time/60), round(mod(avg_time, 60)));
    fprintf([del_string, message])
    del_string = repmat(sprintf('\b'), 1, length(message));

    % Perform smoothing
%     SSM                         = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
    states_MC                   = simulation_smoother(Y, SSM_med);
    GDP_solera(:, :, i_MC)      = states_MC(1:N_c, :);
    states_MC                   = simulation_smoother(Y_last_signal, SSM_med);
    GDP_last_signal(:, :, i_MC) = states_MC(1:N_c, :);    
    states_MC                   = simulation_smoother(Y_last_series, SSM_med);
    GDP_last_series(:, :, i_MC) = states_MC(1:N_c, :);
    
    % Compute execution time
    avg_time = toc*(n_MC-i_MC)/(i_MC+1);        
end

% Compute smoothed growth rates
growth_solera      = [NaN(1, n_MC); 4*reshape(GDP_solera(N_c, 2:T, :) - GDP_solera(N_c, 1:(T-1), :), [T-1, n_MC])];
quant_solera       = quantile(growth_solera, [0.05, 0.5, 0.95], 2);
growth_last_signal = [NaN(1, n_MC); 4*reshape(GDP_last_signal(N_c, 2:T, :) - GDP_last_signal(N_c, 1:(T-1), :), [T-1, n_MC])];
quant_last_signal  = quantile(growth_last_signal, [0.05, 0.5, 0.95], 2);
growth_last_series = [NaN(1, n_MC); 4*reshape(GDP_last_series(N_c, 2:T, :) - GDP_last_series(N_c, 1:(T-1), :), [T-1, n_MC])];
quant_last_series  = quantile(growth_last_series, [0.05, 0.5, 0.95], 2);

% Define recent period
recent      = (datetime(2014, 1, 1) <= time) & (time < datetime(2017, 1, 1));
time_recent = time(recent);


% Create table with GDP solera estimates
table_solera   = array2table([year(time), quarter(time), round(quantile(growth_solera, [0.05, 0.50, 0.95], 2), 2), ...
                                                         round(exp(quantile(reshape(GDP_solera(N_c, :, :), [T, n_MC]), [0.05, 0.50, 0.95], 2)/100), 2)], ...
    'variablenames', {'Year', 'Quarter', 'GrowthLow', 'GrowthMid', 'GrowthUpp', 'LevelLow', 'LevelMed', 'LevelUpp'});
display(table_solera)
writetable(table_solera, [tab_dir sprintf('solera%s.csv', fname)], 'writerownames', true, 'writevariablenames', true)


% Create figure comparing GDP solera with last-signal estimate
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], [quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')], ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
fill(ax0, [time_recent', fliplr(time_recent')], [quant_last_signal(recent, 1)', fliplr(quant_last_signal(recent, 3)')], ...
    dark_green, 'facealpha', 0.1, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time_recent, quant_solera(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', '-')
set(plot0, 'color', dark_green)
hold('all')
plot0 = plot(ax0, time_recent, quant_last_signal(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', ':')
set(plot0, 'color', dark_green)
hold('off')

% Add extras
xlim(ax0, [time_recent(1), time_recent(end)])
xticks(ax0, time_recent(3:3:12))
xticklabels(ax0, {'2013Q2', '2014Q1', '2014Q4', '2015Q3'})
ylim(ax0, [-1, 6]);
ylabel(ax0, 'GDP growth estimates')
line(ax0, [time_recent(1), time_recent(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(ax0, 'All', 'Last signal only', 'Location', 'south', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'historical_last_signal_2015.eps'], '-depsc')


% Create figure comparing GDP solera with last-series estimate
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], [quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')], ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
fill(ax0, [time_recent', fliplr(time_recent')], [quant_last_series(recent, 1)', fliplr(quant_last_series(recent, 3)')], ...
    dark_green, 'facealpha', 0.1, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time_recent, quant_solera(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', '-')
set(plot0, 'color', dark_green)
hold('all')
plot0 = plot(ax0, time_recent, quant_last_series(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', ':')
set(plot0, 'color', dark_green)
hold('off')

% Add extras
xlim(ax0, [time_recent(1), time_recent(end)])
xticks(ax0, time_recent(3:3:12))
xticklabels(ax0, {'2013Q2', '2014Q1', '2014Q4', '2015Q3'})
ylim(ax0, [-1, 6]);
ylabel(ax0, 'GDP growth estimates')
line(ax0, [time_recent(1), time_recent(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(ax0, 'All', 'Last series only', 'Location', 'south', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'historical_last_series_2015.eps'], '-depsc')


% Define recent period
recent      = (datetime(2017, 1, 1) <= time) & (time < datetime(2020, 1, 1));
time_recent = time(recent);


% Create figure comparing GDP solera with last-signal estimate
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], [quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')], ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
fill(ax0, [time_recent', fliplr(time_recent')], [quant_last_signal(recent, 1)', fliplr(quant_last_signal(recent, 3)')], ...
    dark_green, 'facealpha', 0.1, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time_recent, quant_solera(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', '-')
set(plot0, 'color', dark_green)
hold('all')
plot0 = plot(ax0, time_recent, quant_last_signal(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', ':')
set(plot0, 'color', dark_green)
hold('off')

% Add extras
xlim(ax0, [time_recent(1), time_recent(end)])
xticks(ax0, time_recent(3:3:12))
xticklabels(ax0, {'2017Q2', '2018Q1', '2018Q4', '2019Q3'})
ylim(ax0, [-1, 6]);
ylabel(ax0, 'GDP growth estimates')
line(ax0, [time_recent(1), time_recent(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(ax0, 'All', 'Last signal only', 'Location', 'south', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'historical_last_signal_2018.eps'], '-depsc')


% Create figure comparing GDP solera with last-series estimate
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], [quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')], ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
fill(ax0, [time_recent', fliplr(time_recent')], [quant_last_series(recent, 1)', fliplr(quant_last_series(recent, 3)')], ...
    dark_green, 'facealpha', 0.1, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time_recent, quant_solera(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', '-')
set(plot0, 'color', dark_green)
hold('all')
plot0 = plot(ax0, time_recent, quant_last_series(recent, 2));
set(plot0, 'linewidth', 2)
set(plot0, 'linestyle', ':')
set(plot0, 'color', dark_green)
hold('off')

% Add extras
xlim(ax0, [time_recent(1), time_recent(end)])
xticks(ax0, time_recent(3:3:12))
xticklabels(ax0, {'2017Q2', '2018Q1', '2018Q4', '2019Q3'})
ylim(ax0, [-1, 6]);
ylabel(ax0, 'GDP growth estimates')
line(ax0, [time_recent(1), time_recent(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(ax0, 'All', 'Last series only', 'Location', 'south', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'historical_last_series_2018.eps'], '-depsc')


% Create figure comparing width of pointwise intervals
interval_w = [quant_solera(:, 3)-quant_solera(:, 1), ...
              quant_last_signal(:, 3)-quant_last_signal(:, 1), ...
              quant_last_series(:, 3)-quant_last_series(:, 1)];
fig0  = figure();
ax0   = axes();
plot0 = plot(ax0, time, interval_w);
set(plot0, 'linewidth', 2)
set(plot0, {'linestyle'}, {'-'; ':'; '-.'})
set(plot0, {'color'}, {dark_green; green; black})

% Add extras
xlim(ax0, [time(1), time(end)])
ylim(ax0, [0, 1.05*max(interval_w(:))]);
ylabel(ax0, 'Width of 90% intervals')
legend(ax0, 'All', 'Last signal only', 'Last series only', 'Location', 'south', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')
set(ax0, 'linewidth', 0.5)

% Save and close figure
print(fig0, [fig_dir 'historical_widths.eps'], '-depsc')


%% PLOT OF GDP SOLERA OVER 2019-2023

% Transform geometric to arithmetic rates
quant_solera = quantile(growth_solera, [0.05, 0.5, 0.95], 2);
quant_solera = 100*(exp(quant_solera/100) - 1);

% Set period for plot
recent      = (2019 <= year(time)) & (year(time) <= 2023);
time_recent = time(recent);
disp('Growth')
disp([quant_solera(recent, :), growth_GDE(recent), growth_GDI(recent)])

% Create figure comparing GDP solera with last-series estimate
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], [quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')], ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time_recent, [quant_solera(recent, 2), growth_GDE(recent), growth_GDI(recent)]);
hold('off')

% Change x-axis to quarters
xlim(ax0, [datetime(2019, 1, 1), datetime(2023, 1, 1)])
xticks(ax0, datetime(2019, 4+6*(0:7), 1))
xticklabels(ax0, {'2019Q2', '2019Q4', '2020Q2', '2020Q4', '2021Q2', '2021Q4', '2022Q2', '2022Q4'})
ylim(ax0, [-37, 30]);
ylabel(ax0, 'GDPsolera (green), GDE (blue) and GDI (red)')
line(ax0, [time_recent(1), time_recent(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(plot0, {'GDPsolera', 'GDE', 'GDI'}, 'Location', 'southeast', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, {'color'}, {dark_green; blue; red}) 
set(plot0, {'linewidth'}, {2; 1.5; 1.5})
set(plot0, {'linestyle'}, {'-'; '-.'; '-.'})

% Save and close figure
print(fig0, [fig_dir 'solera_2019_2023_diff.eps'], '-depsc')


% Arrange data to plot
Y_GDE     = Y(1:N_E, :);
Y_GDI     = Y(N_E+(1:N_I), :);
Y_GDE_aux = Y_GDE(any(~isnan(Y_GDE(:, recent)), 2), recent);
Y_GDI_aux = Y_GDI(any(~isnan(Y_GDI(:, recent)), 2), recent);
Y_GDE_aux = Y_GDE_aux(end:(-1):1, :);
Y_GDI_aux = Y_GDI_aux(end:(-1):1, :);
quant_solera = quantile(reshape(GDP_solera(N_c, :, :), [T, n_MC]), [0.05, 0.5, 0.95], 2); % redefined for levels
disp('Levels')
disp(exp([quant_solera(recent, :), Y_GDE_aux', Y_GDI_aux']/100))

% Create figure for zoomed-in data
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], exp([quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')]/100), ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time(recent), exp([Y_GDE_aux', Y_GDI_aux', quant_solera(recent, 2)]/100));
hold('off')

% Create style and color property cells
lwidth  = {1.2; 1.2; 1.2; 1.2; 1.2; 1.2; ...
                1.2; 1.2; 1.2; 1.2; 1.2; ...
             2};
colors  = {blue; blue; blue; blue; blue; blue; ...
                  red;  red;  red;  red;  red; ...
           dark_green};
styles  = {'-'; ':'; '-.'; '--'; '-'; '-'; ...
           '-'; ':'; '-.'; '--'; '-'; ...
           '-'};
markers = {'s'; 'x'; 'd'; '+'; 'o'; 'p'; ...
           's'; 'x'; 'd'; '+'; 'o'; ...
           'none'};
       
% Change x-axis to quarters
xlim(ax0, [datetime(2018, 12, 1), datetime(2023, 3, 1)])
xticks(ax0, datetime(2019, 4+6*(0:7), 1))
xticklabels(ax0, {'2019Q2', '2019Q4', '2020Q2', '2020Q4', '2021Q2', '2021Q4', '2022Q2', '2022Q4'})
       
% Add extras
ylabel(ax0, 'GDPsolera (green), GDE (blue) and GDI (red)')
try
    legend(plot0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                   'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'GDPsolera'}, ...
                   'location', 'SouthEast', 'NumColumns', 2) %#ok
catch
    legend(plot0, {'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                   'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'GDPsolera'}, ...
                   'location', 'best')    
end

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, 'linewidth', 2)
set(plot0, {'linewidth'}, lwidth)
set(plot0, {'linestyle'}, styles)
set(plot0, {'color'}, colors)
set(plot0, {'marker'}, markers)
set(plot0, 'markersize', 8)

% Save figure
print(fig0, [fig_dir 'solera_2019_2023_level.eps'], '-depsc')


%% PLOT OF GDP SOLERA OVER 1984-2023

% Transform geometric to arithmetic rates
quant_solera = quantile(growth_solera, [0.05, 0.5, 0.95], 2);
quant_solera = 100*(exp(quant_solera/100) - 1);

% Set period for plot
recent      = (1985 <= year(time)) & (year(time) <= 2023);
time_recent = time(recent);
disp('Growth')
disp([quant_solera(recent, :), growth_GDE(recent), growth_GDI(recent)])

% Create figure comparing GDP solera with last-series estimate
fig0  = figure();
ax0   = axes();
fill(ax0, [time_recent', fliplr(time_recent')], [quant_solera(recent, 1)', fliplr(quant_solera(recent, 3)')], ...
    dark_green, 'facealpha', 0.3, 'linestyle', 'none');
hold('all')
plot0 = plot(ax0, time_recent, [quant_solera(recent, 2), growth_GDE(recent), growth_GDI(recent)]);
hold('off')

% Change x-axis to quarters
xlim(ax0, [datetime(1985, 1, 1), datetime(2023, 1, 1)])
ylim(ax0, [-37, 30]);
ylabel(ax0, 'GDPsolera (green), GDE (blue) and GDI (red)')
line(ax0, [time_recent(1), time_recent(end)], [0, 0], 'linewidth', 0.5, 'linestyle', '--', 'color', [0, 0, 0])
legend(plot0, {'GDPsolera', 'GDE', 'GDI'}, 'Location', 'southeast', 'orientation', 'horizontal') 

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, {'color'}, {dark_green; blue; red}) 
set(plot0, {'linewidth'}, {2; 1.5; 1.5})
set(plot0, {'linestyle'}, {'-'; '-.'; '-.'})

% Save and close figure
print(fig0, [fig_dir 'solera_1984_2023_diff.eps'], '-depsc')


% Arrange data to plot
Y_GDE     = Y(1:N_E, :);
Y_GDI     = Y(N_E+(1:N_I), :);
Y_GDE_aux = Y_GDE(any(~isnan(Y_GDE(:, recent)), 2), recent);
Y_GDI_aux = Y_GDI(any(~isnan(Y_GDI(:, recent)), 2), recent);
Y_GDE_aux = Y_GDE_aux(end:(-1):1, :);
Y_GDI_aux = Y_GDI_aux(end:(-1):1, :);
quant_solera = quantile(permute(GDP_solera(1:N_c, :, :), [2, 3, 1]), [0.05, 0.5, 0.95], 2); % redefined for levels
disp('Levels')
disp(exp([quant_solera(recent, :), Y_GDE_aux', Y_GDI_aux']/100))

% Create figure for zoomed-in data
fig0  = figure();
ax0   = axes();
for i_c = 1:N_c
    fill(ax0, [time_recent', fliplr(time_recent')], exp([quant_solera(recent, 1, i_c)', fliplr(quant_solera(recent, 3, i_c)')]/100), ...
        dark_green, 'facealpha', 0.3, 'linestyle', 'none');
    hold('all')
end
plot0 = plot(ax0, time(recent), exp([Y_GDE_aux', Y_GDI_aux', squeeze(quant_solera(recent, 2, :))]/100));
hold('off')

% Create style and color property cells
lwidth  = {1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; ...
                1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; 1.2; ...
             2; 2; 2; 2; 2};
colors  = {blue; blue; blue; blue; blue; blue; blue; blue; blue; blue; blue; ...
                  red;  red;  red;  red;  red;  red;  red;  red;  red;  red; ...
           dark_green; dark_green; dark_green; dark_green; dark_green};
styles  = {'-'; '-'; '-'; '-'; '-'; '--'; ':'; '-.'; '--'; ':'; '-.'; ...
           '-'; '-'; '-'; '-'; '-'; '--'; ':'; '-.'; '--'; ':'; ...
           '-'; '-'; '-'; '-'; '-'};
       
% Change x-axis to quarters
xlim(ax0, [datetime(1984, 12, 1), datetime(2022, 12, 1)])
       
% Add extras
ylabel(ax0, 'GDPsolera (green), GDE (blue) and GDI (red)')
try
    legend(plot0, {'compr 5', 'compr 4', 'compr 3', 'compr 2', 'compr 1', 'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                   'compr 5', 'compr 4', 'compr 3', 'compr 2', 'compr 1', 'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'GDPsolera 1', 'GDPsolera 2', 'GDPsolera 3', 'GDPsolera 4', 'GDPsolera 5'}, ...
                   'location', 'NorthWest', 'NumColumns', 3) %#ok
catch
    legend(plot0, {'compr 5', 'compr 4', 'compr 3', 'compr 2', 'compr 1', 'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'advance', ...
                   'compr 5', 'compr 4', 'compr 3', 'compr 2', 'compr 1', 'annual 3', 'annual 2', 'annual 1', 'third', 'second', 'GDPsolera 1', 'GDPsolera 2', 'GDPsolera 3', 'GDPsolera 4', 'GDPsolera 5'}, ...
                   'location', 'best')    
end

% Tune ax handle
set(ax0, ax_fonts{:})
set(ax0, 'box', 'on')

% Tune plot
set(plot0, 'linewidth', 2)
set(plot0, {'linewidth'}, lwidth)
set(plot0, {'linestyle'}, styles)
set(plot0, {'color'}, colors)

% Save figure
print(fig0, [fig_dir 'solera_1984_2023_level.eps'], '-depsc')


%% PLOTS OF PRECISION GAINS

% Compute unconditional standard errors of measurements
SD_E  = NaN(N_E, n_MC);
SD_I  = NaN(N_I, n_MC);
for i_MC = 1:n_MC
    [~, ~, ~, rho_E, Sigma_E, ...
        rho_I, Sigma_I] = map_parameter(param_Gibbs(:, i_MC), dims);
    SD_E(:, i_MC) = sqrt(abs(diag(Sigma_E) ./ (1-rho_E.^2)));
    SD_I(:, i_MC) = sqrt(abs(diag(Sigma_I) ./ (1-rho_I.^2)));
end
SD_E = quantile(SD_E, [0.05, 0.5, 0.95], 2);
SD_I = quantile(SD_I, [0.05, 0.5, 0.95], 2);

% Order unconditional standard errors of measurements
GDE_curve           = NaN(111, 3, 4);
GDE_curve(:, :, 1)  = [           SD_E(1, :); min(SD_E(1:2, :), [], 1); min(SD_E(1:3, :), [], 1).*ones(13, 1); min(SD_E(1:4, :), [], 1).*ones(12, 1); min(SD_E(1:5, :), [], 1).*ones(12, 1); min(SD_E(1:6, :), [], 1).*ones(12, 1); min(SD_E([1:6, 11], :), [], 1).*ones(60, 1)];
GDE_curve(:, :, 2)  = [NaN(3, 3); SD_E(1, :); min(SD_E(1:2, :), [], 1); min(SD_E(1:3, :), [], 1).*ones(10, 1); min(SD_E(1:4, :), [], 1).*ones(12, 1); min(SD_E(1:5, :), [], 1).*ones(12, 1); min(SD_E(1:6, :), [], 1).*ones(12, 1); min(SD_E([1:6, 11], :), [], 1).*ones(60, 1)];
GDE_curve(:, :, 3)  = [NaN(6, 3); SD_E(1, :); min(SD_E(1:2, :), [], 1); min(SD_E(1:3, :), [], 1).*ones(7, 1);  min(SD_E(1:4, :), [], 1).*ones(12, 1); min(SD_E(1:5, :), [], 1).*ones(12, 1); min(SD_E(1:6, :), [], 1).*ones(12, 1); min(SD_E([1:6, 11], :), [], 1).*ones(60, 1)];
GDE_curve(:, :, 4)  = [NaN(9, 3); SD_E(1, :); min(SD_E(1:2, :), [], 1); min(SD_E(1:3, :), [], 1).*ones(4, 1);  min(SD_E(1:4, :), [], 1).*ones(12, 1); min(SD_E(1:5, :), [], 1).*ones(12, 1); min(SD_E(1:6, :), [], 1).*ones(12, 1); min(SD_E([1:6, 11], :), [], 1).*ones(60, 1)];
GDI_curve           = NaN(111, 3, 4);
GDI_curve(:, :, 1)  = [NaN(1, 3);  SD_I(1, :); min(SD_I(1:2, :), [], 1).*ones(13, 1); min(SD_I(1:3, :), [], 1).*ones(12, 1); min(SD_I(1:4, :), [], 1).*ones(12, 1); min(SD_I(1:5, :), [], 1).*ones(12, 1); min(SD_E([1:5, 10], :), [], 1).*ones(60, 1)];
GDI_curve(:, :, 2)  = [NaN(4, 3);  SD_I(1, :); min(SD_I(1:2, :), [], 1).*ones(10, 1); min(SD_I(1:3, :), [], 1).*ones(12, 1); min(SD_I(1:4, :), [], 1).*ones(12, 1); min(SD_I(1:5, :), [], 1).*ones(12, 1); min(SD_I([1:5, 10], :), [], 1).*ones(60, 1)];
GDI_curve(:, :, 3)  = [NaN(7, 3);  SD_I(1, :); min(SD_I(1:2, :), [], 1).*ones(7, 1);  min(SD_I(1:3, :), [], 1).*ones(12, 1); min(SD_I(1:4, :), [], 1).*ones(12, 1); min(SD_I(1:5, :), [], 1).*ones(12, 1); min(SD_I([1:5, 10], :), [], 1).*ones(60, 1)];
GDI_curve(:, :, 4)  = [NaN(11, 3);             min(SD_I(1:2, :), [], 1).*ones(4, 1);  min(SD_I(1:3, :), [], 1).*ones(12, 1); min(SD_I(1:4, :), [], 1).*ones(12, 1); min(SD_I(1:5, :), [], 1).*ones(12, 1); min(SD_I([1:5, 10], :), [], 1).*ones(60, 1)];

% Define vintage indicators
base_year    = 2018; %%%%%%%
n_year       = 4;
n_quarter    = 4;
n_estimate   = 3;
i_c          = N_c;
time_idx     = find(time == datetime(base_year, 1, 1), 1, 'first')-1 + (1:4);
MSE_level    = NaN(4, n_MC, n_estimate, n_quarter, n_year);
MSE_growth   = NaN(4, n_MC, n_estimate, n_quarter, n_year);
del_string   = '';
avg_time     = NaN;
tic
for i_year = 1:n_year
    for i_quarter = 1:n_quarter
        for i_estimate = 1:n_estimate 
            message = sprintf('Year: %d - Quarter: %d - Estimate: %d\n', i_year, i_quarter, i_estimate);
            fprintf([del_string, message])
            del_string = repmat('\b', 1, length(message));

            % Construct data with available measurements
            ref_vintage = vintage2date( sprintf('Y%dQ%dE%d', base_year-1+i_year, i_quarter, i_estimate) );
            mat_vintage = (date_vintage <= ref_vintage);
            Y_vintage   = NaN(N_E+N_I, T);
            Y_vintage(mat_vintage) = 0;

            % Run smoother
            for i_MC = 1:20:n_MC
                param = param_Gibbs(:, i_MC);
                SSM   = construct_SSM(param, delta, dims, mu);
                [~, ~, MSEs] = fast_smoother(Y_vintage, SSM);
                MSE_level(:, i_MC, i_estimate, i_quarter, i_year)  = sqrt(MSEs.states(i_c, i_c, time_idx));
                MSE_growth(:, i_MC, i_estimate, i_quarter, i_year) = 4*sqrt(MSEs.states(i_c, i_c, time_idx) - 2*MSEs.states(i_c, N_c+i_c, time_idx) + MSEs.states(N_c+i_c, N_c+i_c, time_idx));
            end            
        end
    end
end

% Obtain posterior quantiles of MSE curves (levels)
MSE_curve   = reshape(quantile(permute(MSE_level, [3, 4, 5, 2, 1]), [0.05, 0.5, 0.95], 4), [n_estimate*n_quarter*n_year, 3, 4]);
month_curve = datetime(0, 3+(1:(n_estimate*n_quarter*n_year)), 1)';
GDE_curve   = GDE_curve(1:length(month_curve), :, :);
GDI_curve   = GDI_curve(1:length(month_curve), :, :);

for i_quarter = 1:4

    % Define auxiliary vectors for shaded areas
    month_flip = [month_curve', fliplr(month_curve')];
    MSE_flip   = [MSE_curve(:, 1, i_quarter)', fliplr(MSE_curve(:, 3, i_quarter)')];
    GDE_flip   = [GDE_curve(:, 1, i_quarter)', fliplr(GDE_curve(:, 3, i_quarter)')];
    GDI_flip   = [GDI_curve(:, 1, i_quarter)', fliplr(GDI_curve(:, 3, i_quarter)')];

    % Execute and format plot
    fig0  = figure();
    ax0   = axes();
    plot1 = fill(ax0, month_flip, MSE_flip, dark_green, 'facealpha', 0.3);
    set(plot1, 'linestyle', 'none')
    hold('on')
    plot2 = fill(ax0, month_flip(~isnan(GDE_flip)), GDE_flip(~isnan(GDE_flip)), blue, 'facealpha', 0.3);
    set(plot2, 'linestyle', 'none')
    hold('on')
    plot3 = fill(ax0, month_flip(~isnan(GDI_flip)), GDI_flip(~isnan(GDI_flip)), red, 'facealpha', 0.3);
    set(plot3, 'linestyle', 'none')
    hold('on')
    plot0 = plot(ax0, month_curve, [MSE_curve(:, 2, i_quarter), GDE_curve(:, 2, i_quarter), GDI_curve(:, 2, i_quarter)]);
    hold('off')
    
    % Add extras
    xlim(ax0, [month_curve(max(1, 3*(i_quarter-1))), month_curve(find(month(month_curve)==11, 1, 'last'))])
    ylim(ax0, [0, 1])
    ylabel(ax0, 'RMSE for estimate of GDP level')
    for i_quart = (i_quarter-1):(4*n_year-1)
        line(ax0, [(month_curve(max(1, 3*i_quart))+28) (month_curve(max(1, 3*i_quart))+28)], [0, 1], 'linestyle', '--', 'color', [0, 0, 0])
    end

    % Tune ax handle
    set(ax0, ax_fonts{:})
    set(ax0, 'box', 'on')
    set(ax0, 'linewidth', 0.5)

    % Tune plot
    set(plot0, 'linewidth', 2)
    set(plot0, 'linestyle', '-')
    set(plot0, {'color'}, {dark_green; blue; red})
    legend(plot0, {'GDPsolera'; 'GDE'; 'GDI'}, 'Location', 'Best')
    
    % Save and close figure
    print(fig0, [fig_dir sprintf('precision_gains_level_full_%d.eps', i_quarter)], '-depsc')

end


for i_quarter = 1:4

    % Define auxiliary vectors for shaded areas
    month_flip = [month_curve', fliplr(month_curve')];
    MSE_flip   = [MSE_curve(:, 1, i_quarter)', fliplr(MSE_curve(:, 3, i_quarter)')];

    % Execute and format plot
    fig0  = figure();
    ax0   = axes();
    plot1 = fill(ax0, month_flip, MSE_flip, dark_green, 'facealpha', 0.3);
    set(plot1, 'linestyle', 'none')
    hold('on')
    plot0 = plot(ax0, month_curve, MSE_curve(:, 2, i_quarter));
    hold('off')
    
    % Add extras
    xlim(ax0, [month_curve(max(1, 3*(i_quarter-1))), month_curve(find(month(month_curve)==11, 1, 'last'))])
    ylim(ax0, [0, 1])
    ylabel(ax0, 'RMSE for estimate of GDP level')
    for i_quart = (i_quarter-1):(4*n_year-1)
        line(ax0, [(month_curve(max(1, 3*i_quart))+28) (month_curve(max(1, 3*i_quart))+28)], [0, 1], 'linestyle', '--', 'color', [0, 0, 0])
    end

    % Tune ax handle
    set(ax0, ax_fonts{:})
    set(ax0, 'box', 'on')
    set(ax0, 'linewidth', 0.5)

    % Tune plot
    set(plot0, 'linewidth', 2)
    set(plot0, 'linestyle', '-')
    set(plot0, 'color', dark_green)
    legend(plot0, {'GDPsolera'}, 'Location', 'Best')
    
    % Save and close figure
    print(fig0, [fig_dir sprintf('precision_gains_level_%d.eps', i_quarter)], '-depsc')

end


% Obtain posterior quantiles of MSE curves (growth rates)
MSE_curve = reshape(quantile(permute(MSE_growth, [3, 4, 5, 2, 1]), [0.05, 0.5, 0.95], 4), [n_estimate*n_quarter*n_year, 3, 4]);
month_curve = datetime(0, 3+(1:(n_estimate*n_quarter*n_year)), 1)';

for i_quarter = 1:4

    % Execute and format plot
    fig0  = figure();
    ax0   = axes();
    plot1 = fill(ax0, [month_curve', fliplr(month_curve')], [MSE_curve(:, 1, i_quarter)', fliplr(MSE_curve(:, 3, i_quarter)')], dark_green, 'facealpha', 0.3);
    set(plot1, 'linestyle', 'none')
    hold('all')
    plot0 = plot(ax0, month_curve, MSE_curve(:, 2, i_quarter));

    % Add extras
    xlim(ax0, [month_curve(max(1, 3*(i_quarter-1))), month_curve(find(month(month_curve)==11, 1, 'last'))])
    ylim(ax0, [0, 2.5])
    ylabel(ax0, 'RMSE for estimate of GDP growth')
    for i_quart = (i_quarter-1):(4*n_year-1)
        line(ax0, [(month_curve(max(1, 3*i_quart))+28) (month_curve(max(1, 3*i_quart))+28)], [0, 2.5], 'linestyle', '--', 'color', [0, 0, 0])
    end

    % Tune ax handle
    set(ax0, ax_fonts{:})
    set(ax0, 'box', 'on')
    set(ax0, 'linewidth', 0.5)

    % Tune plot
    set(plot0, 'linewidth', 2)
    set(plot0, 'linestyle', '-')
    set(plot0, 'color', dark_green)
    
    % Save and close figure
    print(fig0, [fig_dir sprintf('precision_gains_diff_%d.eps', i_quarter)], '-depsc')

end


%% LOCAL FUNCTIONS

% Compute LRV using parametric VAR model
function LRV = compute_LRV(Y, nLag)

[N, T] = size(Y);
if (nargin < 2), nLag = ceil(0.75*(T^(1/3))); end
y      = Y(:, (nLag+1):T);
x      = [ones(1, T-nLag); NaN(N*nLag, T-nLag)];
for iLag = 1:nLag
    x((2+N*(iLag-1)):(1+N*iLag), :) = Y(:, (nLag+1-iLag):(T-iLag));
end
B      = y/x;
U      = y - B*x;
S      = U*(U')/((T-nLag)-(1+N*nLag));
IB     = pinv(eye(N) - sum(reshape(B(:, 2:(1+N*nLag)), [N, N, nLag]), 3));
LRV    = IB*S*(IB');

end

% Compute dimensions of subplot axes
function [nRow, nColumn] = compute_dimensions(nParam)

nRowMax    = 10;
nColumnMax = 13;
matrixTemp = ((1:nRowMax)') + (1:nColumnMax);
matrixTemp(((1:nRowMax)') .* (1:nColumnMax) < nParam) = Inf;
matrixTemp(((1:nRowMax)') - ((1+sqrt(5))/2)*(1:nColumnMax) > 0) = Inf;
[matrixTemp, nRow] = min(matrixTemp, [], 1);
[~, nColumn]       = min(matrixTemp);
nRow               = nRow(1, nColumn);

end
