%%% GDP SOLERA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set filename
fname = '';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];

% Set code parameters
rng(1914)
settings           = struct();
n_MC               = 2e4;
settings.n_MC      = n_MC;
settings.n_burn    = n_MC;
settings.n_thin    = 5;
settings.plot_MCMC = true;
settings.plot_each = 200;

% Load and arrange data
datasets  = load([data_dir 'data.mat']);
time      = datetime(datenum(datasets.GDE.time, 'yyyyQQ'), 'convertfrom', 'datenum');
subsample = (datetime(1984, 1, 1) <= time) & (time < datetime(2026, 1, 1));
time      = time(subsample, :); 
Y_GDE     = 100*log( [datasets.GDE.early(subsample, :), ...
                      datasets.GDE.late(subsample, :), ...
                      datasets.GDE.compr(subsample, :)] )';
Y_GDI     = 100*log( [datasets.GDI.early(subsample, :), ...
                      datasets.GDI.late(subsample, :), ...
                      datasets.GDI.compr(subsample, :)] )';
Y         = [Y_GDE; Y_GDI];

% Compute an store dimensions in a vector
N_c  = length(datasets.GDE.compr_aux);
N_E  = size(Y_GDE, 1);
N_I  = size(Y_GDI, 1);
T    = nnz(subsample);
dims = [N_c, N_E, N_I, T];
                   
% Define vintage indicators    
date_vintage = vintage2date( [datasets.GDE.early_vintage(subsample, :), ...
                              datasets.GDE.late_vintage(subsample, :), ...
                              datasets.GDE.compr_vintage(subsample, :), ...
                              datasets.GDI.early_vintage(subsample, :), ...
                              datasets.GDI.late_vintage(subsample, :), ...
                              datasets.GDI.compr_vintage(subsample, :)]' );

% Define signal indicators
delta_signal = NaN(N_E+N_I, T);
date_signal  = vintage2date( datasets.GDE.compr_aux );
for i_c = 1:(N_c-1)
    delta_signal(date_vintage >= date_signal(i_c) & date_vintage < date_signal(i_c+1)) = i_c;
end
delta_signal(date_vintage >= date_signal(N_c)) = N_c;
delta_signal = fillmissing(delta_signal, 'next', 2);     % impute missing entries of old signals using next available
delta_signal = fillmissing(delta_signal, 'previous', 2); % impute missing entries of new/discontinued signals using previous available

% Store indicators in 3D array
delta = zeros(N_E+N_I, N_c, T);
for i_c = 1:N_c
    delta(:, i_c, :) = double(delta_signal == i_c);
end


%% GIBBS SAMPLING

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

% Set random number generator seed
rng(1116)

% Construct data with available measurements
ref_vintage = time(end) + calmonths(12);
mat_vintage = (date_vintage <= ref_vintage);
Y_vintage   = NaN(N_E+N_I, T);
Y_vintage(mat_vintage) = Y(mat_vintage);

% Avoid pandemic quarters for estimation
Y_vintage(:, (year(time) == 2020 & quarter(time) == 2)) = NaN;
Y_vintage(:, (year(time) == 2020 & quarter(time) == 3)) = NaN;

% Run Gibbs sampler
param_Gibbs = Gibbs_sampler(Y_vintage, delta, prior, settings);

% Save results
results              = struct();
results.settings     = settings;
results.time         = time;
results.date_vintage = date_vintage;
results.Y            = Y;
results.dims         = dims;
results.delta        = delta;
results.prior        = prior;
results.param_Gibbs  = param_Gibbs;
save([mat_dir sprintf('estimates_full_sample%s.mat', fname)], '-struct', 'results')
