%%% REAL TIME PARALLEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all

% Set filename
fname = '';

% Add paths and directories
addpath(['..' filesep 'simulation' filesep 'v0' filesep 'functions'])
data_dir = ['..' filesep 'data' filesep 'output' filesep];
mat_dir  = ['output' filesep 'matfiles' filesep];

% Set code parameters
settings           = struct();
n_worker           = 244;
n_MC               = 2e4;
settings.n_MC      = n_MC;
settings.n_burn    = 0.1*n_MC;
settings.n_thin    = 5;
settings.plot_MCMC = false;
settings.plot_each = 200;

% Load and arrange data
datasets  = load([data_dir 'data.mat']);
time      = datetime(datenum(datasets.GDE.time, 'yyyyQQ'), 'convertfrom', 'datenum');
subsample = (datetime(1984, 1, 1) <= time) & (time < datetime(2023, 1, 1));
time      = time(subsample, :);
Y_GDE     = 100*log( [datasets.GDE.early(subsample, :), ...
                      datasets.GDE.late(subsample, :), ...
                      datasets.GDE.compr(subsample, :)] )';
Y_GDI     = 100*log( [datasets.GDI.early(subsample, :), ...
                      datasets.GDI.late(subsample, :), ...
                      datasets.GDI.compr(subsample, :)] )';
Y         = [Y_GDE; Y_GDI];

% Compute an store dimensions in a vector
N_c  = length(datasets.GDE.compr_aux);
N_E  = size(Y_GDE, 1);
N_I  = size(Y_GDI, 1);
T    = nnz(subsample);
dims = [N_c, N_E, N_I, T];
description = name_parameter(dims);
n_param = sum(description.n_param);

% Define vintage indicators    
date_vintage = vintage2date( [datasets.GDE.early_vintage(subsample, :), ...
                              datasets.GDE.late_vintage(subsample, :), ...
                              datasets.GDE.compr_vintage(subsample, :), ...
                              datasets.GDI.early_vintage(subsample, :), ...
                              datasets.GDI.late_vintage(subsample, :), ...
                              datasets.GDI.compr_vintage(subsample, :)]' );

% Define signal indicators
delta_signal = NaN(N_E+N_I, T);
date_signal  = vintage2date( datasets.GDE.compr_aux );
for i_c = 1:(N_c-1)
    delta_signal(date_vintage >= date_signal(i_c) & date_vintage < date_signal(i_c+1)) = i_c;
end
delta_signal(date_vintage >= date_signal(N_c)) = N_c;
delta_signal = fillmissing(delta_signal, 'next', 2);     % impute missing entries of old signals using next available
delta_signal = fillmissing(delta_signal, 'previous', 2); % impute missing entries of new/discontinued signals using previous available

% Store indicators in 3D array
delta = zeros(N_E+N_I, N_c, T);
for i_c = 1:N_c
    delta(:, i_c, :) = double(delta_signal == i_c);
end


%% GIBBS SAMPLING

% Set prior
prior    = construct_prior(dims);
mu       = mean(Y(:, 1), 'omitnan')*ones(N_c, 1);
prior.mu = mu;

% Strenghen prior on persistence and variances of measurement errors
prior.P_E  = 5*prior.P_E;
prior.nu_E = 5*prior.nu_E;
prior.P_I  = 5*prior.P_I;
prior.nu_I = 5*prior.nu_I;

% Set numerical parameters for real-time exercise
base_year  = 2002;
base_month = 9;
n_month    = 244; 
real_time  = datetime(base_year, base_month-1 + (1:n_month)', 1);

% Preallocate parameters and states
n_perc        = 99;
grid_perc     = linspace(1/(n_perc+1), n_perc/(n_perc+1), n_perc);
param_save    = NaN(n_param, 3, n_month);
state_save    = NaN(N_c, T, n_perc, n_month);
state_mean    = NaN(N_c, T, n_month);
forecast_mean = NaN(N_E+N_I, T, n_month);

% Run real-time exercise in parallel          
par_pool = parpool(n_worker);
parfor i_month = 1:n_month

    % Set random number generator seed
    rng(1914)
            
    % Construct data with available measurements
    Y_tmp       = Y;
    ref_vintage = real_time(i_month);
    mat_vintage = (date_vintage <= ref_vintage);
    Y_vintage   = NaN(N_E+N_I, T);
    Y_vintage(mat_vintage) = Y_tmp(mat_vintage);

    % Avoid pandemic quarters for estimation
    Y_vintage_est = Y_vintage;
    Y_vintage_est(:, (year(time) == 2020 & quarter(time) == 2)) = NaN;
    Y_vintage_est(:, (year(time) == 2020 & quarter(time) == 3)) = NaN;
    
    % Run Gibbs sampler
    try
    param_Gibbs = Gibbs_sampler(Y_vintage_est, delta, prior, settings);
    catch
        param_Gibbs = zeros(n_param, n_MC);
        disp([num2str(i_month) ' did not converge'])
    end
    param_i     = quantile(param_Gibbs, [0.05, 0.5, 0.95], 2);
    
    % Summarize results
    states_Gibbs = NaN(N_c, T, n_MC);
    for i_MC = 1:n_MC
        SSM       = construct_SSM(param_Gibbs(:, i_MC), delta, dims, mu);
        state_tmp = simulation_smoother(Y_vintage, SSM);
        states_Gibbs(:, :, i_MC) = 4*(state_tmp(1:N_c, :) - state_tmp(N_c + (1:N_c), :));
    end
    state_i = quantile(states_Gibbs, grid_perc, 3);

    % Compute smoothed states
    SSM            = construct_SSM(param_i(:, 2), delta, dims, mu);
    [~, state_tmp] = fast_smoother(Y_vintage, SSM);
    smooth_i       = 4*(state_tmp(1:N_c, :) - state_tmp(N_c + (1:N_c), :));
    forecast_i     = NaN(N_E+N_I, T);
    for t = 1:T
        forecast_i(:, t) = SSM.H(:, :, t)*state_tmp(:, t);
    end
    
    % Save output
    parsave([mat_dir sprintf('results_%d.mat', i_month)], param_i, state_i, smooth_i, forecast_i);
    
end

% Store output
for i_month = 1:n_month
    results = load([mat_dir sprintf('results_%d.mat', i_month)]);
    param_save(:, :, i_month)    = results.param;
    state_save(:, :, :, i_month) = results.states;
    state_mean(:, :, i_month)    = results.smoothed;
    forecast_mean(:, :, i_month) = results.forecast;
    delete([mat_dir sprintf('results_%d.mat', i_month)])
end
results               = struct();
results.settings      = settings;
results.time          = time;
results.real_time     = real_time;
results.date_vintage  = date_vintage;
results.Y             = Y;
results.dims          = dims;
results.delta         = delta;
results.prior         = prior;
results.param_save    = param_save;
results.state_save    = state_save;
results.state_mean    = state_mean;
results.forecast_mean = forecast_mean;
save(sprintf([mat_dir sprintf('estimates_real_time%s.mat', fname)]), '-struct', 'results')
delete(par_pool);


%% LOCAL FUNCTIONS

% Save results during parallel loop
function parsave(fname, param, states, smoothed, forecast)
    res          = struct();
    res.param    = param;
    res.states   = states;
    res.smoothed = smoothed;
    res.forecast = forecast;
    save(fname, '-struct', 'res')
end
