function L = log_prior(param, prior)
% LOG_PRIOR Compute joint log-density of parameters (prior). 
% 
%   L = LOG_PRIOR(PARAM, PRIOR) returns log-density value L for a vector
%   of parameters PARAM and prior PRIOR.
% 
%   Version: 2020 Aug 06 - Tincho Almuzara - Matlab R2017b

% Recover dimensions and set function handles
N_c        = size(prior.s2_X, 1);
N_E        = size(prior.s2_E, 1);
N_I        = size(prior.s2_I, 1);
dims       = [N_c, N_E, N_I];
symmetrize = @(A) (A + A')/2;

% Extract parameters
[mu_X, rho_X, lambda_X, sigma_X, ...
       rho_E, lambda_E, sigma_E, ...
       rho_I, lambda_I, sigma_I] = map_parameter(param, dims);

% Preallocate log-density
L = 0;

% Compute log-density of parameters of signal
beta_X = [mu_X; rho_X; lambda_X];
MatX   = diag(sigma_X) * ((eye(N_c) - (1/(1+N_c))*ones(N_c)) \ diag(sigma_X));
L      = L + sum( gamlogpdf(1./(sigma_X.^2), prior.nu_X/2, 2./(prior.nu_X*prior.s2_X)) ); 
L      = L + mvnlogpdf(beta_X, prior.m_X, symmetrize( kron(pinv(prior.P_X), MatX) ));

% Compute log-density of parameters of measurement error in GDE
MatE   = diag(sigma_E) * ((eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E)) \ diag(sigma_E));
beta_E = [rho_E; lambda_E]; 
L      = L + sum( gamlogpdf(1./(sigma_E.^2), prior.nu_E/2, 2./(prior.nu_E*prior.s2_E)) );
L      = L + mvnlogpdf(beta_E, prior.m_E, symmetrize( kron(pinv(prior.P_E), MatE) ));

% Compute log-density of parameters of measurement error in GDI
MatI   = diag(sigma_I) * ((eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I)) \ diag(sigma_I));
beta_I = [rho_I; lambda_I]; 
L      = L + sum( gamlogpdf(1./(sigma_I.^2), prior.nu_I/2, 2./(prior.nu_I*prior.s2_I)) );
L      = L + mvnlogpdf(beta_I, prior.m_I, symmetrize( kron(pinv(prior.P_I), MatI) ));

end


%% LOCAL FUNCTIONS

function p = gamlogpdf(x, a, b)
    x = x(:);
    a = a(:);
    b = b(:);
    p = -gammaln(a)-a.*log(b)+(a-1).*log(x)-x./b;
end

function p = mvnlogpdf(x, mu, Sigma)
    x  = x(:);
    mu = mu(:);
    n  = size(x(:), 1);
    p  = -(n/2)*log(2*pi) - (1/2)*log(det(Sigma)) - (x-mu)'*(Sigma\(x-mu))/2;
end