function [param, states, ML] = Gibbs_sampler(Y, delta, prior, settings)
% GIBBS_SAMPLER  Gibbs sampler for GDP solera model. Model is described in 
% construct_SSM.m and prior distribution in construct_prior.m.
%
%   PARAM = GIBBS_SAMPLER(Y, DELTA, PRIOR, SETTINGS) returns par's PARAM
%   based on data Y, signal indicators DELTA, prior distrib PRIOR and 
%   numerical settings SETTINGS:
%     Y is NxT, contains data.
%     DELTA is NxN_c (w/N=N_E+N_I), contains signal indicators.
%     PRIOR is struct, see construct_prior.m.
%     SETTINGS is struct:
%     - SETTINGS.n_MC is number of updates to be stored.
%     - SETTINGS.n_burn is number of initial updated burnt-in.
%     - SETTINGS.n_thin is number of updates between stored.
%     - SETTINGS.plot_MCMC is logical, indicates if plot of stored updates
%       is to be displayed, defaults to false.
%     - SETTINGS.plot_each is number of stored updates between plot update.
%     PARAM is (3*N_c+N_c^2 + 3*N_E + 3*N_I)xn_MC, see vec_parameters.m.
%
%   [PARAM, STATES] = GIBBS_SAMPLER(Y, DELTA, PRIOR, SETTINGS) also returns
%   draws of states:
%     STATES is MxTxn_MC (w/M=2*N_c+N_E+N_I+3).
%
%   Version: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b

% Determine tasks
need_states = (nargout > 1);

% Set code parameters and dimensions
n_MC   = settings.n_MC;
n_burn = settings.n_burn;
n_thin = settings.n_thin;
if isfield(settings, 'plot_MCMC') 
    plot_MCMC = settings.plot_MCMC;
    plot_each = settings.plot_each;
else
    plot_MCMC = false;
end
    
% Recover dimensions
N_c = size(prior.s2_X, 1);
N_E = size(prior.s2_E, 1);
N_I = size(prior.s2_I, 1);
T   = size(Y, 2);

% Initialize matrices containing parameter draws
param = NaN(3*N_c+N_c^2 + 3*N_E + 3*N_I, n_MC);
if need_states, states = NaN(2*N_c+1+N_E+1+N_I+1, T, n_MC); end

% Initialize sampler
mu_X     = prior.m_X(1:N_c);
rho_X    = prior.m_X(N_c + (1:N_c^2));
lambda_X = prior.m_X(N_c+N_c^2 + (1:N_c));
sigma_X  = sqrt(prior.s2_X);
rho_E    = prior.m_E(1:N_E);
lambda_E = prior.m_E(N_E + (1:N_E));
sigma_E  = sqrt(prior.s2_E);
rho_I    = prior.m_I(1:N_I);
lambda_I = prior.m_I(N_I + (1:N_I));
sigma_I  = sqrt(prior.s2_I);
param_0  = vec_parameter(mu_X, rho_X, lambda_X, sigma_X, ...
                               rho_E, lambda_E, sigma_E, ...
                               rho_I, lambda_I, sigma_I);

                           
%% GIBBS SAMPLING

% Initialize progress
del_string = '';
avg_time   = NaN;
tic
if plot_MCMC
        
    % Set colors
    blue            = [57, 169, 232]/255;
    red             = [255, 99, 71]/255;              
    golden          = [255, 168, 32]/255;
    property1       = cell(3*N_c+N_c^2, 2); 
    property1(:, 1) = {1.5};
    for i = 1:(N_c+N_c^2),         property1{i, 2} = blue; end
    for i = (N_c+N_c^2)+(1:N_c),   property1{i, 2} = red; end
    for i = (2*N_c+N_c^2)+(1:N_c), property1{i, 2} = golden; end
    property2 = cell(3*N_E ,2);
    property2(:, 1) = {1.5};
    for i = 1:N_E,                 property2{i, 2} = blue; end
    for i = (N_E+1):(2*N_E),       property2{i, 2} = red; end
    for i = (2*N_E+1):(3*N_E),     property2{i, 2} = golden; end
    property3 = cell(3*N_I, 2);
    property3(:, 1) = {1.5};
    for i = 1:N_I,                 property3{i, 2} = blue; end
    for i = (N_I+1):(2*N_I),       property3{i, 2} = red; end
    for i = (2*N_I+1):(3*N_I),     property3{i, 2} = golden; end

    % Begin figures
    ax1  = subplot(1, 3, 1); 
    ax2  = subplot(1, 3, 2);
    ax3  = subplot(1, 3, 3);
    drawnow

end

for i_MC = (-n_burn):n_MC
    
    % Report progress
    message    = sprintf('Gibbs sampler -- Draw: %d/%d - Time left: %d mins %02d secs\n', i_MC, n_MC, floor(avg_time/60), round(mod(avg_time, 60)));
    fprintf([del_string, message])
    del_string = repmat(sprintf('\b'), 1, length(message));

    % Run Gibbs-sampling updates
    for i_thin = 1:n_thin
        [param_0, state] = Gibbs_update(param_0, Y, delta, prior);    
    end

    % Store draws
    if (i_MC > 0)
        param(:, i_MC) = param_0;        
        if need_states, states(:, :, i_MC) = state; end
    end
    
    % Update figures
    if (i_MC > 0) && plot_MCMC && (mod(i_MC, plot_each) == 0)
        plot1 = plot(ax1, param(1:(3*N_c+N_c^2), :)');
        plot2 = plot(ax2, param((3*N_c+N_c^2+1):(3*N_c+N_c^2+3*N_E), :)');
        plot3 = plot(ax3, param((3*N_c+N_c^2+3*N_E+1):(3*N_c+N_c^2+3*N_E+3*N_I), :)');
        ylabel(ax1, 'Parameters of x_t')
        ylabel(ax2, 'Parameters of v_{Et}')
        ylabel(ax3, 'Parameters of v_{It}')
        set(plot1, {'linewidth', 'color'}, property1)
        set(plot2, {'linewidth', 'color'}, property2)
        set(plot3, {'linewidth', 'color'}, property3)
        drawnow
    end    
    
    % Compute execution time
    avg_time = toc*(n_MC-i_MC)/(i_MC+n_burn+1);            
end

% Delete progress report when finished
fprintf(del_string)


%% COMPUTE MARGINAL LIKELIHOOD

if (nargout > 2)
    % Fix parameters at posterior median and compute prior term
    param_med = median(param, 2);
    L_prior   = log_prior(param_med, prior);
    
    % Compute likelihood term
    dims         = [N_c, N_E, N_I, T];
    if isfield(prior, 'mu'), mu = prior.mu; else, mu = mean(Y(:, 1), 'omitnan')*ones(N_c, 1); end
    SSM_med      = construct_SSM(param_med, delta, dims, mu);
    L_likelihood = Kalman_filter(Y, SSM_med);
    
    % Compute posterior term
    L_posterior = NaN(ceil(n_MC/4), 1);
    for i_MC = 1:ceil(n_MC/4)
        states_med        = simulation_smoother(Y, SSM_med);
        L_posterior(i_MC) = log_posterior(param_med, states_med, prior);
    end
        
    % Store output from marginal likelihood calculations
    ML              = struct();
    ML.L_prior      = L_prior;
    ML.L_likelihood = L_likelihood;
    ML.L_posterior  = L_posterior;
end

end