%%% GEWEKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform Geweke's (2004, JASA) posterior simulator test.
%
% Information: 2023 Jul 12 - Tincho Almuzara - Matlab R2017b
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clear memory
clear
clc
close all
addpath('functions')

% Set code parameters
rng(1914)
n_MC         = 2e4;
significance = 0.01;

% Set design
N_c  = 5;
N_E  = N_c + 1;
N_I  = N_c + 1;
T    = 20;
dims = [N_c, N_E, N_I, T];

% Construct prior distribution
prior      = construct_prior(dims);
prior.P_X  = 5*prior.P_X;
prior.nu_X = 5*prior.nu_X;
prior.P_E  = 5*prior.P_E;
prior.nu_E = 5*prior.nu_E;
prior.P_I  = 5*prior.P_I;
prior.nu_I = 5*prior.nu_I;
mu         = 10*ones(N_c, 1);

% Construct measurement equation
delta = NaN(N_E+N_I, N_c, T);
for t = 1:T
    delta(:, :, t) = [binornd(1, 0.5, N_E-N_c, N_c); eye(N_c); ...
                      binornd(1, 0.5, N_I-N_c, N_c); eye(N_c)];
end


%% MARGINAL-CONDITIONAL SIMULATOR

% Create storage for MC draws
param_MC   = NaN(3*N_c+N_c^2+3*N_E+3*N_I, n_MC);
del_string = '';

for i_MC = 1:n_MC
    % Report progress
    message    = sprintf('  Marginal-Conditional simulator -- Draw: %d/%d\n', i_MC, n_MC);
    fprintf([del_string, message])
    del_string = repmat(sprintf('\b'), 1, length(message));

    % Draw from marginal-conditional simulator    
    param = simulate_prior(prior);
    
    % Store draw
    param_MC(:, i_MC) = param;
end


%% SUCCESSIVE-CONDITIONAL SIMULATOR

% Create storage for SC draws
param_SC   = NaN(3*N_c+N_c^2+3*N_E+3*N_I, n_MC);
del_string = '';

for i_MC = 1:n_MC
    % Report progress
    message    = sprintf('Successive-Conditional simulator -- Draw: %d/%d\n', i_MC, n_MC);
    fprintf([del_string, message])
    del_string = repmat(sprintf('\b'), 1, length(message));

    % Draw from successive-conditional simulator
    SSM   = construct_SSM(param, delta, dims, mu);
    y     = simulate_SSM(SSM, T);
    param = Gibbs_update(param, y, delta, prior);

    % Store draw
    param_SC(:, i_MC) = param;  
end


%% TEST STATISTICS

description = name_parameter(dims);

% Compute test functions
g      = @(z) abs(z);
G_MC   = g(param_MC);
G_SC   = g(param_SC);
G      = G_MC - G_SC;
n_G    = size(G, 1);
mean_G = mean(G, 2);

% Compute statistics and tests
var_G      = cov(G_SC') + compute_LRV(G_SC);
stat_ind_G = sqrt(n_MC)*mean_G./sqrt(diag(var_G));
pval_ind_G = 1-chi2cdf(stat_ind_G.^2, 1);
test_ind_G = (pval_ind_G <= significance);

% Construct table
testTable = array2table(round([test_ind_G, stat_ind_G, pval_ind_G, mean(param_MC, 2), mean(param_SC, 2), std(param_MC, [], 2), std(param_SC, [], 2)], 2), ...
    'rownames', description.param_names_tab, 'variablenames', {'test'; 'statistic'; 'pvalue'; 'meanMC'; 'meanSC'; 'stdMC'; 'stdSC'});
display(testTable)
writetable(testTable, ['output' filesep 'tables' filesep 'geweke.xlsx'], 'WriteRowNames', false, 'WriteVariableNames', true)

% Construct plots
for i_param = 1:length(description.param_groupnames)
    
    n_G_tmp           = description.n_param(i_param);
    n_G_cumulat       = sum(description.n_param(1:(i_param-1)), 2);
    fig0              = figure();
    [n_row, n_column] = compute_dimensions(n_G_tmp);

    for i_G = 1:n_G_tmp       
        % Compute plot data
        abscisa = linspace(quantile([param_MC(n_G_cumulat+i_G, :), param_SC(n_G_cumulat+i_G, :)]', 0.01), ...
            quantile([param_MC(n_G_cumulat+i_G, :), param_SC(n_G_cumulat+i_G, :)]', 0.99), 1e2)';
        dens_MC = ksdensity(param_MC(n_G_cumulat+i_G, :)', abscisa);
        dens_SC = ksdensity(param_SC(n_G_cumulat+i_G, :)', abscisa);

        % Execute and format plot
        test_axes = subplot(n_row, n_column, i_G);
        test_plot = plot(test_axes, abscisa, [dens_MC, dens_SC]);
        set(test_axes, 'PlotBoxAspectRatio', [(1+sqrt(5))/2, 1, 1])
        set(test_axes, 'linewidth', 1, 'fontname', 'Palatino', 'fontsize', 12, 'fontweight', 'bold')
        set(test_axes, 'box', 'on')
        axis(test_axes, 'tight')
        ylabel(test_axes, description.param_names_fig{n_G_cumulat+i_G}, 'Interpreter', 'TeX')
        set(test_plot, 'linewidth', 2)
        set(test_plot, {'color'}, {[30, 144, 255]/255; [255, 99, 71]/255})
    end

    print(fig0, ['output' filesep 'figures' filesep sprintf('geweke_%s.eps', description.param_groupnames{i_param})], '-deps')

end


%% LOCAL FUNCTIONS

% Compute LRV using parametric VAR model
function LRV = compute_LRV(Y, nLag)

[N, T] = size(Y);
if (nargin < 2), nLag = ceil(0.75*(T^(1/3))); end
y      = Y(:, (nLag+1):T);
x      = [ones(1, T-nLag); NaN(N*nLag, T-nLag)];
for iLag = 1:nLag
    x((2+N*(iLag-1)):(1+N*iLag), :) = Y(:, (nLag+1-iLag):(T-iLag));
end
B      = y/x;
U      = y - B*x;
S      = U*(U')/((T-nLag)-(1+N*nLag));
IB     = pinv(eye(N) - sum(reshape(B(:, 2:(1+N*nLag)), [N, N, nLag]), 3));
LRV    = IB*S*(IB');

end

% Compute dimensions of subplot axes
function [nRow, nColumn] = compute_dimensions(nParam)

nRowMax    = 8;
nColumnMax = 13;
matrixTemp = ((1:nRowMax)') + (1:nColumnMax);
matrixTemp(((1:nRowMax)') .* (1:nColumnMax) < nParam) = Inf;
matrixTemp(((1:nRowMax)') - ((1+sqrt(5))/2)*(1:nColumnMax) > 0) = Inf;
[matrixTemp, nRow] = min(matrixTemp, [], 1);
[~, nColumn]       = min(matrixTemp);
nRow               = nRow(1, nColumn);

end