function prior = construct_prior(dims)
% CONSTRUCT_PRIOR  Construct prior for parameters of GDP solera model:
%   mu_T, lambda_T| sigma_T   ~ N(m_T, kron(Pi_T, sigma_T.^2)),
%   1/sigma_T.^2              ~ Gamma(nu_X/2, 2./(nu_X*s2_X)),
%   rho_C, lambda_C, sigma_C(:, 1) | sigma_C(:, 2) 
%                             ~ N(m_C, kron(Pi_C, sigma_C.^2)),
%   1/sigma_C(:, 2).^2        ~ Gamma(nu_X/2, 2./(nu_X*s2_X)),
%   rho_E, lambda_E | sigma_E ~ N(m_E, kron(Pi_E, sigma_E.^2)),
%   1/sigma_E.^2              ~ Gamma(nu_E/2, 2./(nu_E*s2_E)),
%   rho_I, lambda_I | sigma_I ~ N(m_I, kron(Pi_I, sigma_I.^2)),
%   1/sigma_I.^2              ~ Gamma(nu_I/2, 2./(nu_I*s2_I)).
% 
%   PRIOR = CONSTRUCT_PRIOR(DIMS) returns prior based on dimensions DIMS:
%     DIMS = [N_C, N_E, N_I, T]:
%     - N_C is number of underlying comprehensive measures.
%     - N_E is number of GDP measurements.
%     - N_I is number of GDI measurements.
%     - T is the time series dimension.
%     PRIOR is struct:
%     - PRIOR.P_T is 2x2, precision of (mu_T, lambda_T).
%     - PRIOR.m_T is 2N_Cx1 mean of (mu_T, lambda_T).
%     - PRIOR.nu_T is degrees of freedom of sigma_T^2.
%     - PRIOR.s2_T is N_Cx1 (approx) mean of sigma_T^2.
%     - PRIOR.P_C is 5x5, precision of (rho_C, lambda_C, sigma_C(:, 1)).
%     - PRIOR.m_C is 5N_Cx1 mean of (rho_C, lambda_C, sigma_C(:, 1)).
%     - PRIOR.nu_C is degrees of freedom of sigma_C(:, 2)^2.
%     - PRIOR.s2_C is N_Cx1 (approx) mean of sigma_C(:, 2)^2.
%     - PRIOR.P_E is 2x2, precision of (rho_E, lambda_E).
%     - PRIOR.m_E is 2N_Ex1 mean of (rho_E, lambda_E).
%     - PRIOR.nu_E is degrees of freedom of sigma_E^2.
%     - PRIOR.s2_E is N_Ex1 (approx) mean of sigma_E^2.
%     - PRIOR.P_I is 2x2, precision of (rho_I, lambda_I).
%     - PRIOR.m_I is 2N_Ix1 mean of (rho_I, lambda_I).
%     - PRIOR.nu_I is degrees of freedom of sigma_I^2.
%     - PRIOR.s2_I is N_Ix1 (approx) mean of sigma_I^2.
% 
%   Version: 2023 Jul 11 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c = dims(1);
N_E = dims(2);
N_I = dims(3);

% Calibrate values from GDP-GDI paper
m_T  =  0.75;
s_T  =  1.20;
r_C  = [1.20, -0.70];
s_C  =  0.70;
s_CT = -0.50;
r_E  =  0.00;
s_E  =  0.10;
r_I  =  0.00;
s_I  =  0.10;

% Calibrate split between common and idiosyncratic variance
w_common_signal = 0.8;
w_common_error  = 0.2;

% Create prior struct
prior = struct();

% Parameters of trend in GDP signal
prior.P_T  = blkdiag(10, 10);
prior.m_T  = [m_T*ones(N_c, 1); ...
              s_T*sqrt(w_common_signal)*ones(N_c, 1)];
prior.nu_T = 10;
prior.s2_T = (s_T^2)*(1-w_common_signal)*ones(N_c, 1);

% Parameters of cycle in GDP signal
prior.P_C  = blkdiag(30, 30, 10, 10, 10);
prior.m_C  = [r_C(1)*ones(N_c, 1); ...
              r_C(2)*ones(N_c, 1); ...
              s_CT*sqrt(w_common_signal)*ones(N_c, 1); ...
              s_C*sqrt(w_common_signal)*ones(N_c, 1); ...
              s_CT*sqrt(1-w_common_signal)*ones(N_c, 1)];
prior.nu_C = 10;
prior.s2_C = (s_C^2)*(1-w_common_signal)*ones(N_c, 1);

% Parameters of measurement error in GDE
prior.P_E  = blkdiag(10, 10);
prior.m_E  = [r_E*ones(N_E, 1); sqrt(s_E)*sqrt(w_common_error)*ones(N_E, 1)];
prior.nu_E = 20;
prior.s2_E = s_E*(1-w_common_error)*ones(N_E, 1);

% Parameters of measurement error in GDI
prior.P_I  = blkdiag(10, 10);
prior.m_I  = [r_I*ones(N_I, 1); sqrt(s_I)*sqrt(w_common_error)*ones(N_I, 1)];
prior.nu_I = 20;
prior.s2_I = s_I*(1-w_common_error)*ones(N_I, 1);

end