function L = log_posterior(param, states, prior)
% LOG_POSTERIOR Compute joint log-density of parameters (posterior). 
% 
%   L = LOG_POSTERIOR(PARAM, STATE, PRIOR) returns log-density value L for
%   a vector of parameters PARAM, state variables STATES and prior PRIOR. 
% 
%   Version: 2023 Jul 11 - Tincho Almuzara - Matlab R2017b

% Recover dimensions
N_c  = size(prior.s2_T, 1);
N_E  = size(prior.s2_E, 1);
N_I  = size(prior.s2_I, 1);
T    = size(states, 2);
dims = [N_c, N_E, N_I, T];

% Extract parameters
[mu_T, lambda_T, sigma_T, rho_C, lambda_C, sigma_C, ...
          rho_E, lambda_E, sigma_E, rho_I, lambda_I, sigma_I] = map_parameter(param, dims);

% Define function handle and options for inverse algorithm
symmetrize    = @(A) (A + A')/2;
option        = struct();
option.SYM    = true;
option.POSDEF = true;

% Preallocate log-density
L = 0;

%%% Compute log-density of parameters of cycle in signal
sign_T  = sign(sum(lambda_T));
sign_C  = sign(sum(lambda_C(:, 2)));
beta_C  = [rho_C(:); lambda_C(:); sigma_C(:, 1)];
MatC    = eye(N_c) - (1/(1+N_c))*ones(N_c);
DT      = states(1:N_c, 2:T) - states(1:N_c, 1:(T-1));
fT      = states(N_c+1, 2:T);
e_T     = NaN(N_c, T-1);
for t = 1:(T-1), e_T(:, t) = DT(:, t) - mu_T - lambda_T*sign_T*fT(t); end
e_T     = e_T./sigma_T;
C_0     = states(N_c+1+(1:N_c), 2:T);
C_1     = states(N_c+1+(1:N_c), 1:(T-1));
C_2     = states(2*N_c+1+(1:N_c), 1:(T-1));
fC      = states(3*N_c+2, 2:T);
xy_C    = [sum(C_0.*C_1, 2); ...
           sum(C_0.*C_2, 2); ...
           sum(C_0.*fT, 2); ...
           sum(C_0.*fC, 2); ...
           sum(C_0.*e_T, 2)];
xx_C    = [diag(sum(C_1.^2, 2)), diag(sum(C_1.*C_2, 2)), diag(sum(C_1.*fT, 2)), diag(sum(C_1.*fC, 2)), diag(sum(C_1.*e_T, 2)); ...
           diag(sum(C_2.*C_1, 2)), diag(sum(C_2.^2, 2)), diag(sum(C_2.*fT, 2)), diag(sum(C_2.*fC, 2)), diag(sum(C_2.*e_T, 2)); ...
           diag(sum(fT.*C_1, 2)), diag(sum(fT.*C_2, 2)), sum(fT.^2, 2)*eye(N_c), sum(fT.*fC, 2)*eye(N_c), diag(sum(fT.*e_T, 2)); ...
           diag(sum(fC.*C_1, 2)), diag(sum(fC.*C_2, 2)), sum(fC.*fT, 2)*eye(N_c), sum(fC.^2, 2)*eye(N_c), diag(sum(fC.*e_T, 2)); ...
           diag(sum(e_T.*C_1, 2)), diag(sum(e_T.*C_2, 2)), diag(sum(e_T.*fT, 2)), diag(sum(e_T.*fC, 2)), diag(sum(e_T.^2, 2))];
P_C_inv = symmetrize( linsolve(symmetrize( kron(prior.P_C, MatC) + xx_C ), eye(5*N_c), option) );
m_C     = P_C_inv*(kron(prior.P_C, MatC)*prior.m_C + xy_C);
e_C     = NaN(N_c, T-1);
for t = 1:(T-1), e_C(:, t) = C_0(:, t) - rho_C(:, 1).*C_1(:, t) - rho_C(:, 2).*C_2(:, t) - lambda_C(:, 1)*sign_T*fT(t) - lambda_C(:, 2)*sign_C*fC(t) - sigma_C(:, 1).*e_T(:, t); end
nu_C    = prior.nu_C + T-1;
s2_C    = (prior.nu_C/nu_C)*prior.s2_C + (1/nu_C)*sum(e_C.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_C(:, 2).^2), nu_C/2, 2./(nu_C*s2_C)) ); 
L       = L + mvnlogpdf(beta_C, m_C, symmetrize( symmetrize( kron(eye(5), diag(sigma_C(:, 2)))*P_C_inv*kron(eye(5), diag(sigma_C(:, 2))) ) ));

%%% Compute log-density of parameters of trend in signal
beta_T  = [mu_T; lambda_T];
sign_T  = sign(sum(lambda_T(:)));
MatT    = eye(N_c) - (1/(1+N_c))*ones(N_c);
DT      = states(1:N_c, 2:T) - states(1:N_c, 1:(T-1));
fT      = states(N_c+1, 2:T);
xy_T    = [sum(DT, 2); ...
           sum(DT.*fT, 2)];
xx_T    = [(T-1)*eye(N_c),   sum(fT)*eye(N_c); ...
           sum(fT)*eye(N_c), sum(fT.^2)*eye(N_c)];
P_T_inv = symmetrize( linsolve(symmetrize( kron(prior.P_T, MatT) + xx_T ), eye(2*N_c), option) );
m_T     = P_T_inv*(kron(prior.P_T, MatT)*prior.m_T + xy_T);
e_T     = NaN(N_c, T-1);
for t = 1:(T-1), e_T(:, t) = DT(:, t) - mu_T - lambda_T*sign_T*fT(t); end
nu_T    = prior.nu_T + T-1;
s2_T    = (prior.nu_T/nu_T)*prior.s2_T + (1/nu_T)*sum(e_T.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_T.^2), nu_T/2, 2./(nu_T*s2_T)) ); 
L       = L + mvnlogpdf(beta_T, m_T, symmetrize( kron(eye(2), diag(sigma_T))*P_T_inv*kron(eye(2), diag(sigma_T)) ) );

%%% Compute log-density of parameters of measurement error in GDE
beta_E  = [rho_E; lambda_E]; 
sign_E  = sign(sum(lambda_E));
MatE    = eye(N_E) - (0.25/(1+0.25*N_E))*ones(N_E);
vE      = states((3*N_c+2) + (1:N_E), :);
fE      = states(3*N_c+2+N_E+1, :);
xy_E    = [sum(vE(:, 2:T).*vE(:, 1:(T-1)), 2); ...
           sum(vE(:, 2:T).*fE(2:T), 2)];    
xx_E    = [diag( sum(vE(:, 1:(T-1)).^2, 2) ),       diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ); ...
           diag( sum(vE(:, 1:(T-1)).*fE(2:T), 2) ), sum(fE(2:T).^2)*eye(N_E)];
P_E_inv = symmetrize( linsolve(symmetrize(kron(prior.P_E, MatE) + xx_E), eye(2*N_E), option) );
m_E     = P_E_inv*(kron(prior.P_E, MatE)*prior.m_E + xy_E);
e_E     = NaN(N_E, T-1);
for t = 1:(T-1), e_E(:, t) = vE(:, t+1) - rho_E.*vE(:, t) - lambda_E*sign_E*fE(t+1); end
nu_E    = prior.nu_E + T-1;
s2_E    = (prior.nu_E/nu_E)*prior.s2_E + (1/nu_E)*sum(e_E.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_E.^2), nu_E/2, 2./(nu_E*s2_E)) );
L       = L + mvnlogpdf(beta_E, m_E, symmetrize( kron(eye(2), diag(sigma_E))*P_E_inv*kron(eye(2), diag(sigma_E)) ));

%%% Compute log-density of parameters of measurement error in GDI
beta_I  = [rho_I; lambda_I]; 
sign_I  = sign(sum(lambda_I));
MatI    = eye(N_I) - (0.25/(1+0.25*N_I))*ones(N_I);
vI      = states((3*N_c+2+N_E+1) + (1:N_I), :);
fI      = states(3*N_c+2+N_E+1+N_I+1, :);
xy_I    = [sum(vI(:, 2:T).*vI(:, 1:(T-1)), 2); ...
           sum(vI(:, 2:T).*fI(2:T), 2)];
xx_I    = [diag( sum(vI(:, 1:(T-1)).^2, 2) ),       diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ); ...
           diag( sum(vI(:, 1:(T-1)).*fI(2:T), 2) ), sum(fI(2:T).^2)*eye(N_I)];
P_I_inv = symmetrize( linsolve(symmetrize(kron(prior.P_I, MatI) + xx_I), eye(2*N_I), option) );
m_I     = P_I_inv*(kron(prior.P_I, MatI)*prior.m_I + xy_I);
e_I     = NaN(N_I, T-1);
for t = 1:(T-1), e_I(:, t) = vI(:, t+1) - rho_I.*vI(:, t) - lambda_I*sign_I*fI(t+1); end
nu_I    = prior.nu_I + T-1;
s2_I    = (prior.nu_I/nu_I)*prior.s2_I + (1/nu_I)*sum(e_I.^2, 2);
L       = L + sum( gamlogpdf(1./(sigma_I.^2), nu_I/2, 2./(nu_I*s2_I)) );
L       = L + mvnlogpdf(beta_I, m_I, symmetrize( kron(eye(2), diag(sigma_I))*P_I_inv*kron(eye(2), diag(sigma_I)) ));

end


%% LOCAL FUNCTIONS

function p = gamlogpdf(x, a, b)
    x = x(:);
    a = a(:);
    b = b(:);
    p = -gammaln(a)-a.*log(b)+(a-1).*log(x)-x./b;
end

function p = mvnlogpdf(x, mu, Sigma)
    x  = x(:);
    mu = mu(:);
    n  = size(x(:), 1);
    p  = -(n/2)*log(2*pi) - (1/2)*log(det(Sigma)) - (x-mu)'*(Sigma\(x-mu))/2;
end